package net.engio.mbassy.messages;

import java.util.concurrent.atomic.AtomicInteger;

/**
 *
 *
* @author bennidi
* Date: 11/22/12
*/
public class TestMessage {

    public AtomicInteger counter = new AtomicInteger();

}
